% This m-file makes a picture illustrating the Fast marching method
% written by Treb Allen in 2014, revised in May 2016.

% Note: To increase the speed of the program, I highly recommend compiling the accurate fast marching method
% toolbox by Dirk-Jan Kroon (included in the zip file or available from
% download at
% http://www.mathworks.com/matlabcentral/fileexchange/24531-accurate-fast-marching)
% using the "compile_c_files.m".

clear
clc

%% Step #1: Load the image into Matlab, identify key features, assign speed of travel
   
    % Loading a map of Evanston (downloaded from Google maps)
        map = imread('Evanston - Google map.png','png');

    % Identifying the various features of the map using their colors
            train = (double(map)<135);
            water = double(map)==135; 
            parks =(double(map)>=136 & double(map)<=153);
            trails = (double(map)>=154 & double(map)<=158);
            campus = (double(map)==159);
            beach = (double(map)==160);
            bigroads = (double(map)>=161 & double(map)<=170);
            buildings = (double(map)>=171 & double(map)<=209);
            private = (double(map)>=210 & double(map)<=239);
            roads = (double(map)>240);
            
    % Assigning an inverse speed of travel to each of the features (so tau is the number of minutes I assume it would take to walk a mile across a feature)
        tau = 30*ones(size(beach));
        tau = 8*roads + 200*private + 200*buildings + 300*water + 7*bigroads + 35*campus + 25*beach + 20*parks + 100*train + 7*trails;
    
        pix_size = size(map,1) / 1.1; % each side of the image is about 1.1 miles
        
    
    % Here is the speed image
        figure(1)
            clf
            hold on
            imagesc(flipud(tau))
            scatter(260,225,250,'p','y','fill','MarkerEdgeColor','k')
            caxis([8 50])
            colorbar
            axis tight
            axis off
            title('Evanston, IL: Minutes / mile')
            hold off
            
%% Step #2: Choose an origin pixel, and calculate the travel time to all other pixels

    % Here is the origin pixel (my house). Note that you would have to
    % iterate over many origin points to calculate all possible bilateral
    % travel times.
        origin = [225;260];
        
    % Here is the matrix of travel times from that origin, i.e. t(1,1) is
    % the travel time across the speed image from pixel (1,1) to pixel
    % (225,260).
        t = msfm(pix_size./tau, origin, true, true);
        
    % Here is a contour map illustrating the result.
        figure(2)
            clf
            hold on
            %imagesc(t)
            image(flipdim(imread('Evanston - Google map.png'),1))
            contour(flipud(t),[0:1:20])
            scatter(origin(2),origin(1),250,'p','y','fill','MarkerEdgeColor','k')
            caxis([0,20])
            colorbar
            axis tight
            axis off
            title('Evanston, IL: Walking time')
            hold off
